=== modified file 'bzrlib/bundle/serializer/v10.py'
--- bzrlib/bundle/serializer/v10.py	2007-06-14 21:06:48 +0000
+++ bzrlib/bundle/serializer/v10.py	2007-06-14 23:24:59 +0000
@@ -199,6 +199,11 @@
 
     revisions = property(_get_revisions)
 
+    def _get_target(self):
+        return self.revisions[-1].revision_id
+
+    target = property(_get_target)
+
 
 class RevisionInstaller(object):
 
@@ -212,6 +217,7 @@
         current_versionedfile = None
         pending_file_records = []
         added_inv = set()
+        target_revision = None
         for bytes, parents, repo_kind, revision_id, file_id in\
             self._container.iter_records():
             if  repo_kind != 'file':

=== modified file 'bzrlib/multiparent.py'
--- bzrlib/multiparent.py	2007-06-14 21:06:48 +0000
+++ bzrlib/multiparent.py	2007-06-14 23:24:59 +0000
@@ -21,23 +21,25 @@
     descendants = {}
     if versions is None:
         versions = vf.versions()
+    def pending_parents(version):
+        return [v for v in vf.get_parents(version) if v in versions and
+                v not in seen]
     for version_id in versions:
         for parent_id in vf.get_parents(version_id):
             descendants.setdefault(parent_id, []).append(version_id)
-    cur = [v for v in versions if len([v for v in vf.get_parents(v) if
-           v in versions]) == 0]
+    cur = [v for v in versions if len(pending_parents(v)) == 0]
     while len(cur) > 0:
         next = []
         for version_id in cur:
             if version_id in seen:
                 continue
-            parents = vf.get_parents(version_id)
-            if not seen.issuperset(parents):
+            if len(pending_parents(version_id)) != 0:
                 continue
             next.extend(descendants.get(version_id, []))
             yield version_id
             seen.add(version_id)
         cur = next
+    assert len(seen) == len(versions)
 
 
 class MultiParent(object):

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2007-06-14 15:14:50 +0000
+++ bzrlib/tests/test_bundle.py	2007-06-14 23:24:59 +0000
@@ -1053,7 +1053,6 @@
         # This should also validate the generated bundle 
         bundle = read_bundle(bundle_txt)
         repository = self.b1.repository
-        return bundle
         for bundle_rev in bundle.real_revisions:
             # These really should have already been checked when we read the
             # bundle, since it computes the sha1 hash for the revision, which
@@ -1067,8 +1066,8 @@
                                  getattr(bundle_rev, a))
             self.assertEqual(len(branch_rev.parent_ids), 
                              len(bundle_rev.parent_ids))
-        self.assertEqual(rev_ids, 
-                         [r.revision_id for r in bundle.real_revisions])
+        self.assertEqual(set(rev_ids),
+                         set([r.revision_id for r in bundle.real_revisions]))
         self.valid_apply_bundle(base_rev_id, bundle,
                                    checkout_dir=checkout_dir)
 

